from dataclasses import dataclass

from dataclasses_json import dataclass_json

from unipayment.models import Invoice


@dataclass_json
@dataclass
class CreateInvoiceResponse:
    code: str
    msg: str
    data: Invoice



class QueryInvoicesRequest(object):
    def __init__(self, page_no=1, page_size=10, is_asc=True, app_id=None, invoice_id=None, order_id=None, status=None,
                 start=None, end=None):
        self.page_no = page_no
        self.page_size = page_size
        self.is_asc = is_asc
        self.app_id = app_id
        self.invoice_id = invoice_id
        self.order_id = order_id
        self.status = status
        self.start = start
        self.end = end

    def to_str(self):
        query_params = {
            "page_no": self.page_no,
            "page_size": self.page_size,
            "is_asc": str(self.is_asc).lower()
        }
        if self.app_id:
            query_params["app_id"] = self.app_id
        if self.invoice_id:
            query_params["invoice_id"] = self.invoice_id
        if self.order_id:
            query_params["order_id"] = self.order_id
        if self.status:
            query_params["status"] = self.status
        if self.start:
            query_params["start"] = self.start
        if self.end:
            query_params["end"] = self.end
        return query_params



import json

from .base_client import BaseClient
from .models.create_payment_request import CreatePaymentRequest
from .models.get_payment_fee_response import GetPaymentFeeResponse
from .models.payment_note import PaymentNote
from .models.payment_response import PaymentResponse
from .models.query_payments_request import QueryPaymentsRequest
from .models.query_payments_response import QueryPaymentsResponse


class PaymentAPI(BaseClient):
    pass

    def create_payment(self, create_payment_request: CreatePaymentRequest) -> PaymentResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/payments'
        response_text = self.call_api(url, 'POST', body=json.loads(create_payment_request.to_json()))
        return PaymentResponse.from_json(response_text)

    def get_payment_by_id(self, payment_id) -> PaymentResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/payments/{payment_id}'
        response_text = self.call_api(url, 'GET')
        return PaymentResponse.from_json(response_text)

    def query_payments(self,
                       query_payments_request: QueryPaymentsRequest = None) -> QueryPaymentsResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/payments'
        if query_payments_request is None:
            query_payments_request = QueryPaymentsRequest()
        response_text = self.call_api(url, 'GET', query_params=query_payments_request.to_str())
        return QueryPaymentsResponse.from_json(response_text)

    def confirm_payment(self, payment_id, payment_note: PaymentNote) -> PaymentResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/payments/{payment_id}/confirm'
        response_text = self.call_api(url, 'PUT', body=json.loads(payment_note.to_json()))
        return PaymentResponse.from_json(response_text)

    def cancel_payment(self, payment_id, payment_note: PaymentNote) -> PaymentResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/payments/{payment_id}/cancel'
        response_text = self.call_api(url, 'PUT', body=json.loads(payment_note.to_json()))
        return PaymentResponse.from_json(response_text)

    def get_payment_fee(self, asset_type) -> GetPaymentFeeResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/payments/fee'
        response_text = self.call_api(url, 'GET', query_params={"asset_type": asset_type})
        return GetPaymentFeeResponse.from_json(response_text)



from dataclasses import dataclass
from typing import Optional, List

from dataclasses_json import dataclass_json

from unipayment.models import DepositAddress


@dataclass_json
@dataclass
class GetDepositAddressResponse:
    code: str
    msg: str
    data: Optional[List[DepositAddress]] = None



from __future__ import absolute_import

import json

from .base_client import BaseClient
from .models import Beneficiary, PaymentMethod, PaymentMethodResponse, QueryPaymentMethodsResponse, \
    BeneficiaryResponse, QueryBeneficiariesRequest, QueryBeneficiariesResponse


class BeneficiaryAPI(BaseClient):
    pass

    def create_beneficiary(self, beneficiary: Beneficiary) -> BeneficiaryResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries'
        response_text = self.call_api(url, 'POST', body=json.loads(beneficiary.to_json()))
        return BeneficiaryResponse.from_json(response_text)

    def get_beneficiary_by_id(self, beneficiary_id) -> BeneficiaryResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}'
        response_text = self.call_api(url, 'GET')
        return BeneficiaryResponse.from_json(response_text)

    def update_beneficiary_by_id(self, beneficiary_id, beneficiary: Beneficiary) -> BeneficiaryResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}'
        response_text = self.call_api(url, 'PUT', body=json.loads(beneficiary.to_json()))
        return BeneficiaryResponse.from_json(response_text)

    def delete_beneficiary_by_id(self, beneficiary_id) -> BeneficiaryResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}'
        response_text = self.call_api(url, 'DELETE')
        return BeneficiaryResponse.from_json(response_text)

    def query_beneficiaries(self,
                            query_beneficiaries_request: QueryBeneficiariesRequest = None) -> QueryBeneficiariesResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries'

        if query_beneficiaries_request is None:
            query_beneficiaries_request = QueryBeneficiariesRequest()

        response_text = self.call_api(url, 'GET', query_params=query_beneficiaries_request.to_str())
        return QueryBeneficiariesResponse.from_json(response_text)

    def create_payment_method(self, beneficiary_id,
                              payment_method: PaymentMethod) -> PaymentMethodResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}/payment-methods'
        response_text = self.call_api(url, 'POST', body=json.loads(payment_method.to_json()))
        return PaymentMethodResponse.from_json(response_text)

    def update_payment_method(self, beneficiary_id, payment_method_id,
                              payment_method: PaymentMethod) -> PaymentMethodResponse:
        url = (f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}/payment'
               f'-methods/{payment_method_id}')
        response_text = self.call_api(url, 'PUT', body=json.loads(payment_method.to_json()))
        return PaymentMethodResponse.from_json(response_text)

    def get_payment_method_by_id(self, beneficiary_id, payment_method_id) -> PaymentMethodResponse:
        url = (f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}/payment'
               f'-methods/{payment_method_id}')
        response_text = self.call_api(url, 'GET')
        return PaymentMethodResponse.from_json(response_text)

    def query_payment_methods(self, beneficiary_id) -> QueryPaymentMethodsResponse:
        url = (f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}/payment'
               f'-methods')
        response_text = self.call_api(url, 'GET')
        return QueryPaymentMethodsResponse.from_json(response_text)

    def delete_payment_method_by_id(self, beneficiary_id, payment_method_id) -> PaymentMethodResponse:
        url = (f'{self.configuration.host}/v{self.configuration.api_version}/beneficiaries/{beneficiary_id}/payment'
               f'-methods/{payment_method_id}')
        response_text = self.call_api(url, 'DELETE')
        return PaymentMethodResponse.from_json(response_text)



from .base_client import BaseClient
from .models.get_deposit_address_response import GetDepositAddressResponse
from .models.get_deposit_bank_account_response import GetDepositBankAccountResponse
from .models.get_wallet_accounts_response import GetWalletAccountsResponse
from .models.get_wallet_balances_response import GetWalletBalancesResponse
from .models.query_wallet_account_transactions_request import QueryWalletAccountTransactionsRequest
from .models.query_wallet_account_transactions_response import QueryWalletAccountTransactionsResponse


class WalletAPI(BaseClient):
    pass

    def get_balances(self) -> GetWalletBalancesResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/wallet/balances'
        response_text = self.call_api(url, 'GET')
        return GetWalletBalancesResponse.from_json(response_text)

    def get_accounts(self) -> GetWalletAccountsResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/wallet/accounts'
        response_text = self.call_api(url, 'GET')
        return GetWalletAccountsResponse.from_json(response_text)

    def query_transactions(self, account_id,
                           query_wallet_transactions_request: QueryWalletAccountTransactionsRequest = None) \
            -> QueryWalletAccountTransactionsResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/wallet/accounts/{account_id}/transactions'

        if query_wallet_transactions_request is None:
            query_wallet_transactions_request = QueryWalletAccountTransactionsRequest()

        response_text = self.call_api(url, 'GET', query_params=query_wallet_transactions_request.to_str())
        # Adjust the JSON string to match dataclass field names as 'from' is a reserved keyword
        adjusted_json_string = response_text.replace('"from":', '"from_currency":').replace('"to":', '"to_currency":')
        return QueryWalletAccountTransactionsResponse.from_json(adjusted_json_string)

    def get_deposit_address(self, account_id) -> GetDepositAddressResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/wallet/accounts/{account_id}/deposit/address'
        response_text = self.call_api(url, 'GET')
        return GetDepositAddressResponse.from_json(response_text)

    def get_deposit_bank_account(self, account_id) -> GetDepositBankAccountResponse:
        url = (f'{self.configuration.host}/v{self.configuration.api_version}/wallet/accounts/{account_id}/deposit/bank'
               f'-account')
        response_text = self.call_api(url, 'GET')
        return GetDepositBankAccountResponse.from_json(response_text)



from __future__ import absolute_import

import logging
import uuid

import pytest

from test.test_base_client import TestBaseClient
from unipayment.models import PaymentNote, CreatePaymentRequest, PaymentReason, QueryPaymentsRequest

logger = logging.getLogger(__name__)


class TestPaymentAPI(TestBaseClient):
    pass

    @pytest.mark.order(1)
    def test_create_payment(self):
        """
            Test case create_payment
        """
        """ Set Correct Account IDs & Payment Method ID """
        create_payment_request = CreatePaymentRequest(
            from_account_id='d7c5db2e-8572-4a2f-9300-84dc4b3fd052',
            to_account_id='f0b4083b-8b43-4267-a321-f96bdba8c9e4',
            asset_type='USDT',
            amount=10,
            reason=PaymentReason.INTERNAL_TRANSFER,
            payment_method_id='5c0bce95-7d10-47f3-8e11-250ab900da07',
            unique_id=uuid.uuid4()
        )
        create_payment_response = self.PaymentAPI.create_payment(create_payment_request)
        logger.debug("response body: %s", create_payment_response.to_json())
        self.assertEqual('OK', create_payment_response.code)

    @pytest.mark.order(2)
    def test_get_payment_by_id(self):
        """
            Test case get_payment_by_id
        """

        """ Set Payment ID """
        payment_id = self.create_payment()

        get_payment_response = self.PaymentAPI.get_payment_by_id(payment_id)
        logger.debug("response body: %s", get_payment_response.to_json())
        self.assertEqual('OK', get_payment_response.code)
        self.assertIsNotNone(get_payment_response.data)

    @pytest.mark.order(3)
    def test_confirm_payment(self):
        """
            Test case confirm_payment
        """

        """ Set Payment ID """
        payment_id = self.create_payment()
        payment_note = PaymentNote(note="Payment Confirmed")
        confirm_payment_response = self.PaymentAPI.confirm_payment(payment_id, payment_note)
        logger.debug("response body: %s", confirm_payment_response.to_json())
        self.assertEqual('OK', confirm_payment_response.code)

    @pytest.mark.order(4)
    def test_cancel_payment(self):
        """
            Test case cancel_payment
        """

        """ Set Payment ID """
        payment_id = self.create_payment()
        payment_note = PaymentNote(note="Payment Cancelled")
        cancel_payment_response = self.PaymentAPI.cancel_payment(payment_id, payment_note)
        logger.debug("response body: %s", cancel_payment_response.to_json())
        self.assertEqual('OK', cancel_payment_response.code)

    @pytest.mark.order(5)
    def test_query_payments(self):
        """
            Test case query_payments
        """
        query_payments_request = QueryPaymentsRequest()
        query_payments_response = self.PaymentAPI.query_payments(query_payments_request)
        logger.debug("response body: %s", query_payments_response.to_json())
        self.assertEqual('OK', query_payments_response.code)
        self.assertIsNotNone(query_payments_response.data)

    def test_get_payment_fee(self):
        """
            Test case get_payment_fee
        """

        get_payment_fee_response = self.PaymentAPI.get_payment_fee('USDT')
        logger.debug("response body: %s", get_payment_fee_response.to_json())
        self.assertEqual('OK', get_payment_fee_response.code)
        self.assertIsNotNone(get_payment_fee_response.data)

    def create_payment(self):
        create_payment_request = CreatePaymentRequest(
            from_account_id='d7c5db2e-8572-4a2f-9300-84dc4b3fd052',
            to_account_id='f0b4083b-8b43-4267-a321-f96bdba8c9e4',
            asset_type='USDT',
            amount=10,
            reason=PaymentReason.INTERNAL_TRANSFER,
            payment_method_id='5c0bce95-7d10-47f3-8e11-250ab900da07',
            unique_id=uuid.uuid4()
        )
        create_payment_response = self.PaymentAPI.create_payment(create_payment_request)
        return create_payment_response.data.id



from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json

from unipayment.models import ConfirmSpeed, BuyerInfo


@dataclass_json
@dataclass
class CreateInvoiceRequest:
    app_id: str
    price_amount: float
    price_currency: str
    order_id: str
    pay_currency: Optional[str] = None
    network: Optional[str] = None
    lang: Optional[str] = None
    ext_args: Optional[str] = None
    payment_method_type: Optional[str] = None
    host_to_host_mode: bool = False
    notify_url: Optional[str] = None
    redirect_url: Optional[str] = None
    title: Optional[str] = None
    description: Optional[str] = None
    confirm_speed: Optional[ConfirmSpeed] = None
    buyer_info: Optional[BuyerInfo] = None



import json

from .base_client import BaseClient
from .models.create_invoice_request import CreateInvoiceRequest
from .models.create_invoice_response import CreateInvoiceResponse
from .models.get_invoice_by_id_response import GetInvoiceByIdResponse
from .models.query_invoices_request import QueryInvoicesRequest
from .models.query_invoices_response import QueryInvoicesResponse


class BillingAPI(BaseClient):
    pass

    def create_invoice(self, create_invoice_request: CreateInvoiceRequest) -> CreateInvoiceResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/invoices'
        response_text = self.call_api(url, 'POST', body=json.loads(create_invoice_request.to_json()))
        return CreateInvoiceResponse.from_json(response_text)

    def get_invoice_by_id(self, invoice_id) -> GetInvoiceByIdResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/invoices/{invoice_id}'
        response_text = self.call_api(url, 'GET')
        return GetInvoiceByIdResponse.from_json(response_text)

    def query_invoices(self, query_invoices_request: QueryInvoicesRequest) -> QueryInvoicesResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/invoices'
        response_text = self.call_api(url, 'GET', query_params=query_invoices_request.to_str())
        return QueryInvoicesResponse.from_json(response_text)



import json

from .base_client import BaseClient
from .models.api_response import ApiResponse
from .models.update_notify_url_request import UpdateNotifyURLRequest
from .models.update_secret_key_request import UpdateSecretKeyRequest


class WebhookAPI(BaseClient):
    pass

    def update_notify_url(self, update_notify_url_request: UpdateNotifyURLRequest) -> ApiResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/webhook/notify-url'
        response_text = self.call_api(url, 'POST', body=json.loads(update_notify_url_request.to_json()))
        return ApiResponse.from_json(response_text)

    def update_secret_key(self, update_secret_key_request: UpdateSecretKeyRequest) -> ApiResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/webhook/secret-key'
        response_text = self.call_api(url, 'POST', body=json.loads(update_secret_key_request.to_json()))
        return ApiResponse.from_json(response_text)



from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class BuyerInfo:
    name: Optional[str] = None
    email: Optional[str] = None
    phone: Optional[str] = None
    address1: Optional[str] = None
    address2: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    country: Optional[str] = None
    zip_code: Optional[str] = None



from __future__ import absolute_import

import base64
import json
import logging
from datetime import datetime

from .api_client import ApiClient
from .api_exception import ApiException
from .configuration import Configuration
from .models import TokenResponse
from .token_cache import TokenCache
from .unipayment_sdk_exception import UnipaymentSdkException

LOGGER = logging.getLogger(__name__)


def is_valid(token: str) -> bool:
    if not token:
        return False

    parts = token.split(".")
    if len(parts) != 3:
        return False

    try:
        decoded_part2 = base64.b64decode(parts[1] + "===").decode('utf-8')
        data_map = json.loads(decoded_part2)
        exp = int(data_map.get("exp"))
        exp_in_millis = exp * 1000
        LOGGER.info("Access Token expires on: %s", datetime.fromtimestamp(exp_in_millis / 1000))
        current_millis = int(datetime.utcnow().timestamp() * 1000)
        return exp_in_millis > current_millis
    except (json.JSONDecodeError, ValueError) as e:
        LOGGER.warning("Invalid token: %s", token, exc_info=e)
        return False


class BaseClient(object):
    def __init__(self, configuration: Configuration):
        self.configuration = configuration
        self.api_client = ApiClient(self.configuration)
        self.api_client.set_default_header('Accept', 'application/json')
        self.api_client.set_default_header('Content-Type', 'application/json')
        if configuration.debug:
            LOGGER.setLevel(logging.DEBUG)

    def call_api(self, url, method, query_params=None, post_params=None, body=None):
        access_token = TokenCache.get_access_token()

        if access_token is None or is_valid(access_token) is False:
            token_response = self.get_access_token()
            if token_response is None:
                raise ApiException("Unable to get access token")
            else:
                access_token = token_response.access_token
                TokenCache.set_access_token(access_token, token_response.expires_in)

        headers = self.api_client.default_headers
        headers['Authorization'] = f'Bearer {access_token}'
        response = self.api_client.request(url, method, headers=headers, query_params=query_params,
                                           post_params=post_params, body=body)
        status_code = response.status
        if 200 <= status_code < 300:
            return response.data.decode('utf-8')
        else:
            raise ApiException(http_resp=response)

    def get_access_token(self) -> TokenResponse:
        headers = {'Accept': 'application/json', 'Content-Type': 'application/x-www-form-urlencoded'}
        post_params = {'grant_type': 'client_credentials', 'client_id': self.configuration.client_id,
                       'client_secret': self.configuration.client_secret}

        url = f'{self.configuration.host}/connect/token'
        response_text = self.api_client.request(url, 'POST', headers=headers, post_params=post_params).data.decode(
            'utf-8')
        response_json = json.loads(response_text)
        if 'error' in response_json:
            raise UnipaymentSdkException(message=response_json['error'])
        return TokenResponse.from_json(response_text)



from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class Transaction:
    hash: str
    network: str
    symbol: str
    from_address: str
    to_address: str
    amount: float
    confirmation_count: int
    is_confirmed: Optional[bool] = None



from __future__ import absolute_import

import logging
import multiprocessing
import sys


class Configuration(object):

    def __init__(self):
        """Constructor"""
        self.host = ""
        self.client_id = ""
        self.client_secret = ""
        self.app_id = ""
        self.api_version = "1.0"
        self.is_sandbox = True

        if self.is_sandbox:
            self.host = "https://sandbox-api.unipayment.io"
        else:
            self.host = "https://api.unipayment.io"

        self.logger = {
            "package_logger": logging.getLogger("unipayment_sdk"),
            "urllib3_logger": logging.getLogger("urllib3")
        }
        self.logger_format = '%(asctime)s %(levelname)s %(message)s'
        self.debug = False

        self.verify_ssl = True
        self.ssl_ca_cert = None
        self.cert_file = None
        self.key_file = None
        self.assert_hostname = None

        self.connection_pool_maxsize = multiprocessing.cpu_count() * 5
        self.proxy = None
        self.safe_chars_for_path_param = ''

    def to_debug_report(self):
        return (
            f"Unipayment Python SDK Debug Report:\n"
            f"OS: {sys.platform}\n"
            f"Python Version: {sys.version}\n"
            f"Version of the API: 2.0.0\n"
            f"SDK Package Version: 2.0.0"
        )



class QueryBeneficiariesRequest(object):
    def __init__(self, page_no=1, page_size=10, is_asc=True):
        self.page_no = page_no
        self.page_size = page_size
        self.is_asc = is_asc

    def to_str(self):
        query_params = {
            "page_no": self.page_no,
            "page_size": self.page_size,
            "is_asc": str(self.is_asc).lower()
        }
        return query_params



from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class WalletBalance:
    asset_type: str
    balance: float
    frozen_balance: float
    reversed_balance: float
    available: float
    id: Optional[str] = None



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from unipayment.models import ExchangeOrder


@dataclass_json
@dataclass
class QueryExchangeOrderResponse:
    code: str
    msg: str
    data: ExchangeOrder



from __future__ import absolute_import

import logging

import pytest

from test.test_base_client import TestBaseClient

logger = logging.getLogger(__name__)


class TestCommonAPI(TestBaseClient):
    pass

    @pytest.mark.order(1)
    def test_ping(self):
        """
            Test case for ping
        """
        """
            Use POST HTTP Method
        """
        ping_response = self.CommonAPI.ping(use_post=True)
        logger.debug("response body: %s", ping_response.to_json())
        self.assertEqual('OK', ping_response.code)
        self.assertEqual('pong', ping_response.msg)

        """
            Use GET HTTP Method
        """
        ping_response = self.CommonAPI.ping()
        logger.debug("response body: %s", ping_response.to_json())
        self.assertEqual('OK', ping_response.code)
        self.assertEqual('pong', ping_response.msg)

    @pytest.mark.order(2)
    def test_query_ips(self):
        """
            Test case for query_ips
        """
        query_ips_response = self.CommonAPI.query_ips()
        logger.debug("response body: %s", query_ips_response.to_json())
        self.assertEqual('OK', query_ips_response.code)
        self.assertEqual(['2.2.2.2'], query_ips_response.data)

    @pytest.mark.order(3)
    def test_get_currencies(self):
        """
            Test case for get_currencies
        """
        get_currencies_response = self.CommonAPI.get_currencies()
        logger.debug("response body: %s", get_currencies_response.to_json())
        self.assertEqual('OK', get_currencies_response.code)

    @pytest.mark.order(4)
    def test_get_exchange_rate_by_currency_pair(self):
        """
            Test case for get_exchange_rate_by_currency_pair
        """
        get_exchange_rate_response = self.CommonAPI.get_exchange_rate_by_currency_pair('USD', 'BTC')
        logger.debug("response body: %s", get_exchange_rate_response.to_json())
        self.assertEqual('OK', get_exchange_rate_response.code)

    @pytest.mark.order(5)
    def test_get_exchange_rate_by_fiat_currency(self):
        """
            Test case for get_exchange_rate_by_fiat_currency
        """
        get_exchange_rates_response = self.CommonAPI.get_exchange_rate_by_fiat_currency('USD')
        logger.debug("response body: %s", get_exchange_rates_response.to_json())
        self.assertEqual('OK', get_exchange_rates_response.code)

    @pytest.mark.order(6)
    def test_check_ipn(self):
        """
            Test case for check_ipn
        """
        check_ipn_response = self.CommonAPI.check_ipn(
            {'ipn_type': 'invoice', 'event': 'invoice_created',
             'app_id': 'cee1b9e2-d90c-4b63-9824-d621edb38012',
             'invoice_id': '12wQquUmeCPUx3qmp3aHnd',
             'order_id': 'ORDER_123456', 'price_amount': 2.0,
             'price_currency': 'USD', 'network': None, 'address': None,
             'pay_currency': None, 'pay_amount': 0.0, 'exchange_rate': 0.0,
             'paid_amount': 0.0, 'confirmed_amount': 0.0,
             'refunded_price_amount': 0.0,
             'create_time': '2022-09-14T04:57:54.5599307Z',
             'expiration_time': '2022-09-14T05:02:54.559933Z',
             'status': 'New', 'error_status': 'None',
             'ext_args': 'Merchant Pass Through Data', 'transactions': None,
             'notify_id': 'fd58cedd-67c6-4053-ae65-2f6fb09a7d2c',
             'notify_time': '0001-01-01T00:00:00'})
        logger.debug("response body: %s", check_ipn_response.to_json())
        self.assertEqual('OK', check_ipn_response.code)
        self.assertEqual('IPN is verified.', check_ipn_response.msg)



from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json

from unipayment.models import Currency


@dataclass_json
@dataclass
class GetCurrenciesResponse:
    code: str
    msg: str
    data: List[Currency]



from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from dataclasses_json import dataclass_json, config
from marshmallow import fields


@dataclass_json
@dataclass
class Quote:
    quote_id: str
    from_currency: str
    to_currency: str
    valid_until: datetime = field(
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format='iso')
        )
    )
    requested_amount: Optional[float] = None
    exchange_rate: Optional[float] = None
    gross_amount: Optional[float] = None
    net_amount: Optional[float] = None
    fee_currency: Optional[str] = None
    fee: Optional[float] = None



from dataclasses import dataclass
from enum import Enum
from typing import Optional

from dataclasses_json import dataclass_json


class PaymentReason(Enum):
    INVESTMENTS = "Investments"
    INTERNAL_TRANSFER = "InternalTransfer"
    INVOICE_FOR_SERVICES_OR_GOODS = "InvoiceForServicesOrGoods"
    OPERATING_EXPENSE = "OperatingExpense"
    STAFF_COSTS = "StaffCosts"
    SETTLING_CLIENT_FUNDS = "SettlingClientFunds"
    SETTLING_TRADING_COUNTERPARTY_FUNDS = "SettlingTradingCounterpartyFunds"
    OTHER = "Other"


@dataclass_json
@dataclass
class CreatePaymentRequest:
    from_account_id: str
    asset_type: str
    amount: float
    reason: PaymentReason
    payment_method_id: Optional[str] = None
    to_account_id: Optional[str] = None
    reference: Optional[str] = None
    note: Optional[str] = None
    unique_id: Optional[str] = None



from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Optional

from dataclasses_json import dataclass_json, config
from marshmallow import fields


@dataclass_json
@dataclass
class PaymentDestination:
    network: str
    address: str


class PaymentStatus(Enum):
    PENDING = "PENDING"
    CANCELED = "CANCELED"
    CONFIRMED = "CONFIRMED"
    REJECTED = "REJECTED"
    APPROVED = "APPROVED"
    COMPLETED = "COMPLETED"
    FAILED = "FAILED"


@dataclass_json
@dataclass
class Payment:
    id: str
    customer_id: str
    transfer_method: str
    network: str
    asset_type: str
    from_account_id: str
    destination: PaymentDestination
    amount: float
    create_time: datetime = field(
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format='iso')
        )
    )
    update_time: datetime = field(
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format='iso')
        )
    )
    fee: Optional[float] = None
    total_amount: Optional[float] = None
    reference: Optional[str] = None
    reason: Optional[str] = None
    note: Optional[str] = None
    status: Optional[PaymentStatus] = None



from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json

from unipayment.models import Beneficiary


@dataclass_json
@dataclass
class BeneficiaryResponse:
    code: str
    msg: str
    data: Optional[Beneficiary] = None



from __future__ import absolute_import

import time


class TokenCache:
    _cache = {}
    ACCESS_TOKEN = 'access_token'  # Define the constant

    @staticmethod
    def set_access_token(value, ttl=3600):
        expiry = int(time.time()) + ttl
        TokenCache._cache[TokenCache.ACCESS_TOKEN] = {'expiry': expiry, 'value': value}

    @staticmethod
    def get_access_token():
        cached_item = TokenCache._cache.get(TokenCache.ACCESS_TOKEN)
        if cached_item is None:
            return None

        if time.time() > cached_item['expiry']:
            del TokenCache._cache[TokenCache.ACCESS_TOKEN]
            return None

        return cached_item['value']



class QuoteRequest(object):
    def __init__(self, from_currency: str, to_currency: str, exchange_amount: float):
        self.from_currency = from_currency
        self.to_currency = to_currency
        self.exchange_amount = exchange_amount

    def to_str(self):
        query_params = {
            "from_currency": self.from_currency,
            "to_currency": self.to_currency,
            "exchange_amount": self.exchange_amount
        }
        return query_params



from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class QueryIpsResponse:
    code: str
    msg: str
    data: List[str]



from __future__ import absolute_import

# import models into model package
from .exchange_order import ExchangeOrder
from .accept_quote_response import AcceptQuoteResponse
from .bank_account import BankAccount
from .beneficiary import Beneficiary, BeneficiaryType, Relationship
from .bank_info import BankInfo
from .bank_deposit_address import DepositBankAccount
from .beneficiary_response import BeneficiaryResponse
from .check_ipn_response import CheckIpnResponse
from .payment_method import PaymentMethod, TransferMode, PaymentMethodDetail, BankPaymentMethodDetail, \
    CryptoPaymentMethodDetail, InternalPaymentMethodDetail
from .invoice import Invoice, PaymentMethodType, InvoiceStatus, InvoiceErrorStatus, ConfirmSpeed
from .transaction import Transaction
from .invoice_detail import InvoiceDetail
from .buyer_info import BuyerInfo
from .create_invoice_request import CreateInvoiceRequest
from .create_invoice_response import CreateInvoiceResponse
from .create_payment_request import CreatePaymentRequest, PaymentReason
from .currency import Currency
from .deposit_address import DepositAddress
from .exchange_rate import ExchangeRate
from .get_currencies_response import GetCurrenciesResponse
from .get_deposit_address_response import GetDepositAddressResponse
from .get_deposit_bank_account_response import GetDepositBankAccountResponse
from .get_exchange_rate_response import GetExchangeRateResponse
from .get_exchange_rates_response import GetExchangeRatesResponse
from .get_invoice_by_id_response import GetInvoiceByIdResponse
from .payment_fee import PaymentFee
from .get_payment_fee_response import GetPaymentFeeResponse
from .payment import Payment, PaymentStatus, PaymentDestination
from .payment_method_response import PaymentMethodResponse
from .payment_note import PaymentNote
from .ping_response import PingResponse
from .query_result import QueryResult
from .query_beneficiaries_request import QueryBeneficiariesRequest
from .query_beneficiaries_response import QueryBeneficiariesResponse
from .query_exchange_order_response import QueryExchangeOrderResponse
from .query_exchange_orders_request import QueryExchangeOrdersRequest
from .query_exchange_orders_request import QueryExchangeOrdersRequest
from .query_invoices_request import QueryInvoicesRequest
from .query_invoices_response import QueryInvoicesResponse
from .query_ips_response import QueryIpsResponse
from .query_payments_request import QueryPaymentsRequest
from .query_payments_response import QueryPaymentsResponse
from .query_payment_methods_response import QueryPaymentMethodsResponse
from .query_wallet_account_transactions_request import QueryWalletAccountTransactionsRequest, TransactionType
from .query_wallet_account_transactions_response import QueryWalletAccountTransactionsResponse
from .quote import Quote
from .quote_request import QuoteRequest
from .quote_response import QuoteResponse
from .token_response import TokenResponse
from .wallet_balance import WalletBalance
from .wallet_account import WalletAccount, AccountType
from .get_wallet_balances_response import GetWalletBalancesResponse
from .get_wallet_accounts_response import GetWalletAccountsResponse
from .update_notify_url_request import UpdateNotifyURLRequest
from .update_secret_key_request import UpdateSecretKeyRequest

__all__ = ['AcceptQuoteResponse', 'AccountType', 'BankAccount', 'DepositBankAccount', 'BankInfo',
           'BankPaymentMethodDetail', 'Beneficiary', 'BeneficiaryType', 'Relationship', 'BeneficiaryResponse',
           'CheckIpnResponse', 'BuyerInfo', 'CreateInvoiceRequest', 'CreateInvoiceResponse', 'CreatePaymentRequest',
           'ConfirmSpeed', 'CryptoPaymentMethodDetail', 'Currency', 'DepositAddress', 'ExchangeOrder', 'ExchangeRate',
           'GetCurrenciesResponse', 'GetDepositAddressResponse', 'GetDepositBankAccountResponse',
           'GetExchangeRateResponse', 'GetExchangeRatesResponse', 'GetInvoiceByIdResponse', 'GetPaymentFeeResponse',
           'GetWalletBalancesResponse', 'GetWalletAccountsResponse', 'InternalPaymentMethodDetail', 'Invoice',
           'InvoiceDetail', 'InvoiceErrorStatus', 'InvoiceStatus', 'Payment', 'PaymentFee', 'PaymentMethod',
           'PaymentMethodResponse', 'PaymentMethodType', 'PaymentMethodDetail', 'PaymentReason', 'PaymentStatus',
           'PaymentDestination', 'PaymentNote', 'PingResponse', 'QueryBeneficiariesRequest',
           'QueryBeneficiariesResponse', 'QueryExchangeOrderResponse', 'QueryExchangeOrdersRequest',
           'QueryInvoicesRequest', 'QueryInvoicesResponse', 'QueryIpsResponse', 'QueryPaymentsRequest',
           'QueryPaymentsResponse', 'QueryPaymentMethodsResponse', 'QueryResult',
           'QueryWalletAccountTransactionsRequest', 'QueryWalletAccountTransactionsResponse', 'Quote',
           'QuoteRequest', 'QuoteResponse', 'TokenResponse', 'Transaction', 'TransactionType', 'TransferMode',
           'WalletAccount', 'WalletBalance', 'UpdateNotifyURLRequest', 'UpdateSecretKeyRequest']



from dataclasses import dataclass

from dataclasses_json import dataclass_json, LetterCase


@dataclass_json(letter_case=LetterCase.CAMEL)
@dataclass
class TokenResponse:
    access_token: str
    expires_in: int
    token_type: str
    scope: str



from dataclasses import dataclass, field
from datetime import datetime
from enum import Enum
from typing import Optional

from dataclasses_json import dataclass_json, config
from marshmallow import fields


class PaymentMethodType(Enum):
    CRYPTO = "CRYPTO"
    CARD = "CARD"
    SKRILL = "SKRILL"
    UNKNOWN = "UNKNOWN"


class InvoiceStatus(Enum):
    NEW = "New"
    PAID = "Paid"
    CONFIRMED = "Confirmed"
    COMPLETE = "Complete"
    EXPIRED = "Expired"
    INVALID = "Invalid"


class InvoiceErrorStatus(Enum):
    NONE = "None"
    PAID_LATE = "PaidLate"
    PAID_PARTIAL = "PaidPartial"
    PAID_OVER = "PaidOver"
    MARKED = "Marked"


class ConfirmSpeed(Enum):
    LOW = "Low"
    MEDIUM = "Medium"
    HIGH = "High"


@dataclass_json
@dataclass
class Invoice:
    app_id: str
    invoice_id: str
    payment_method_type: PaymentMethodType
    host_to_host_mode: bool
    order_id: str
    price_amount: float
    price_currency: str
    pay_amount: float
    exchange_rate: float
    paid_amount: float
    refunded_price_amount: float
    create_time: datetime = field(
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format='iso')
        )
    )
    expiration_time: datetime = field(
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format='iso')
        )
    )
    confirm_speed: ConfirmSpeed
    status: InvoiceStatus
    error_status: InvoiceErrorStatus
    invoice_url: str
    network: Optional[str] = None
    address: Optional[str] = None
    pay_currency: Optional[str] = None
    host_to_host_data: dict = None



from __future__ import absolute_import

import logging
import unittest

from unipayment import BillingAPI, BeneficiaryAPI, CommonAPI, ExchangeAPI, PaymentAPI, WalletAPI, WebhookAPI
from unipayment import Configuration

logger = logging.getLogger(__name__)


class TestBaseClient(unittest.TestCase):

    def setUp(self):
        self.configuration = Configuration()
        self.configuration.client_id = '74feb539-ba5a-4ae9-b901-4da4fb539574'
        self.configuration.client_secret = 'BsoRhgqzhR1TYMtwTRYdPxBTvR5rxkW9K'
        self.configuration.app_id = '2a9bd90b-fe95-4659-83cb-04de662fbbac'
        self.configuration.debug = True
        self.CommonAPI = CommonAPI(self.configuration)
        self.BeneficiaryAPI = BeneficiaryAPI(self.configuration)
        self.ExchangeAPI = ExchangeAPI(self.configuration)
        self.WalletAPI = WalletAPI(self.configuration)
        self.PaymentAPI = PaymentAPI(self.configuration)
        self.BillingAPI = BillingAPI(self.configuration)
        self.WebhookAPI = WebhookAPI(self.configuration)

        if self.configuration.debug:
            logger.setLevel(logging.DEBUG)

    def tearDown(self):
        pass



from __future__ import absolute_import

from .base_client import BaseClient
from .models.check_ipn_response import CheckIpnResponse
from .models.get_currencies_response import GetCurrenciesResponse
from .models.get_exchange_rate_response import GetExchangeRateResponse
from .models.get_exchange_rates_response import GetExchangeRatesResponse
from .models.ping_response import PingResponse
from .models.query_ips_response import QueryIpsResponse


class CommonAPI(BaseClient):
    pass

    def ping(self, use_post=False) -> PingResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/ping'
        method = 'GET'
        if use_post:
            method = 'POST'

        response_text = self.call_api(url, method)
        return PingResponse.from_json(response_text)

    def query_ips(self) -> QueryIpsResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/ips'
        response_text = self.call_api(url, 'GET')
        return QueryIpsResponse.from_json(response_text)

    def get_currencies(self) -> GetCurrenciesResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/currencies'
        response_text = self.call_api(url, 'GET')
        return QueryIpsResponse.from_json(response_text)

    def get_exchange_rate_by_currency_pair(self, fiat_currency,
                                           crypto_currency) -> GetExchangeRateResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/rates/{fiat_currency}/{crypto_currency}'
        response_text = self.call_api(url, 'GET')
        # Adjust the JSON string to match dataclass field names as 'from' is a reserved keyword
        adjusted_json_string = response_text.replace('"from":', '"from_currency":').replace('"to":', '"to_currency":')
        return GetExchangeRateResponse.from_json(adjusted_json_string)

    def get_exchange_rate_by_fiat_currency(self, fiat_currency, ) -> GetExchangeRatesResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/rates/{fiat_currency}'
        response_text = self.call_api(url, 'GET')
        # Adjust the JSON string to match dataclass field names as from is a reserved keyword
        adjusted_json_string = response_text.replace('"from":', '"from_currency":').replace('"to":', '"to_currency":')
        return GetExchangeRatesResponse.from_json(adjusted_json_string)

    def check_ipn(self, notify) -> CheckIpnResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/ipn'
        response_text = self.call_api(url, 'POST', body=notify)
        return CheckIpnResponse.from_json(response_text)



from dataclasses import dataclass, field
from datetime import datetime
from typing import List, Optional

from dataclasses_json import dataclass_json, config
from marshmallow import fields

from unipayment.models import PaymentMethodType, ConfirmSpeed, InvoiceStatus, Transaction


@dataclass_json
@dataclass
class InvoiceDetail:
    app_id: str
    invoice_id: str
    payment_method_type: PaymentMethodType
    order_id: str
    price_amount: float
    price_currency: str
    pay_amount: float
    exchange_rate: float
    paid_amount: float
    refunded_price_amount: float
    create_time: datetime = field(
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format='iso')
        )
    )
    expiration_time: datetime = field(
        metadata=config(
            encoder=datetime.isoformat,
            decoder=datetime.fromisoformat,
            mm_field=fields.DateTime(format='iso')
        )
    )
    confirm_speed: ConfirmSpeed
    status: InvoiceStatus
    invoice_url: str
    transactions: List[Transaction]
    network: Optional[str] = None
    address: Optional[str] = None
    pay_currency: Optional[str] = None
    error_status: Optional[str] = None



from dataclasses import dataclass
from enum import Enum
from typing import Optional

from dataclasses_json import dataclass_json

from unipayment.models import BankAccount


class AccountType(Enum):
    DEFAULT = "Default"
    SETTLEMENT = "Settlement"
    BANK = "Bank"


@dataclass_json
@dataclass
class WalletAccount:
    asset_type: str
    balance: float
    frozen_balance: float
    reversed_balance: float
    available: float
    type: AccountType
    id: Optional[str] = None
    friendly_name: Optional[str] = None
    account_number: Optional[str] = None
    bank_account: Optional[BankAccount] = None
    status: Optional[str] = None



import hmac
import hashlib
import base64


class WebhookSignatureUtil:
    @staticmethod
    def is_valid(payload: str, secret_key: str, signature_to_verify: str) -> bool:
        """
        Validates the provided signature against the generated signature.

        :param payload: The payload data to be signed.
        :param secret_key: The secret key used for generating the signature.
        :param signature_to_verify: The signature to verify against.
        :return: Returns True if the signatures match, otherwise False.
        """
        return signature_to_verify == WebhookSignatureUtil.generate_signature(payload, secret_key)

    @staticmethod
    def generate_signature(payload: str, secret_key: str) -> str:
        """
        Generates an HMAC-SHA256 signature and encodes it in Base64.

        :param payload: The payload data to be signed.
        :param secret_key: The secret key used for generating the signature.
        :return: The generated Base64-encoded signature.
        """
        # Create an HMAC-SHA256 hash
        hash_bytes = hmac.new(secret_key.encode(), payload.encode(), hashlib.sha256).digest()

        # Encode the hash in Base64
        return base64.b64encode(hash_bytes).decode()



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from unipayment.models import Invoice
from unipayment.models import QueryResult


@dataclass_json
@dataclass
class QueryInvoicesResponse:
    code: str
    msg: str
    data: QueryResult[Invoice]



from __future__ import absolute_import

import logging

import pytest

from test.test_base_client import TestBaseClient
from unipayment.models import QueryExchangeOrdersRequest, QuoteRequest

logger = logging.getLogger(__name__)


class TestExchangeAPI(TestBaseClient):

    @pytest.mark.order(1)
    def test_get_quote(self):
        """
            Test case get_quote
        """
        quote_request = QuoteRequest(
            from_currency='USDT',
            to_currency='BNB',
            exchange_amount=10.0)

        quote_response = self.ExchangeAPI.get_quote(quote_request)
        logger.debug("response body: %s", quote_response.to_json())
        self.assertEqual('OK', quote_response.code)
        self.assertIsNotNone(quote_response.data.quote_id)

    @pytest.mark.order(2)
    def test_accept_quote(self):
        """
            Test case accept_quote
        """
        """
            Get Quote
        """
        quote_request = QuoteRequest(
            from_currency='USDT',
            to_currency='BNB',
            exchange_amount=10.0)

        quote_response = self.ExchangeAPI.get_quote(quote_request)
        logger.debug("response body: %s", quote_response.to_json())
        self.assertEqual('OK', quote_response.code)

        accept_quote_response = self.ExchangeAPI.accept_quote(quote_response.data.quote_id)
        logger.debug("response body: %s", accept_quote_response.to_json())
        self.assertEqual('OK', accept_quote_response.code)

    @pytest.mark.order(3)
    def test_query_exchange_orders(self):
        """
            Test case query_exchange_orders
        """

        query_exchange_orders_request = QueryExchangeOrdersRequest()
        query_exchange_orders_response = self.ExchangeAPI.query_exchange_orders(
            query_exchange_orders_request)
        logger.debug("response body: %s", query_exchange_orders_response.to_json())
        self.assertEqual('OK', query_exchange_orders_response.code)

    @pytest.mark.order(4)
    def test_query_exchange_order_by_order_id(self):
        """
            Test case query_exchange_order_by_order_id
        """
        query_exchange_orders_response = self.ExchangeAPI.query_exchange_orders(
            QueryExchangeOrdersRequest())
        logger.debug("response body: %s", query_exchange_orders_response.to_json())
        self.assertEqual('OK', query_exchange_orders_response.code)
        """ Set Order ID """
        order_id = query_exchange_orders_response.data.models[0].get("id")
        query_exchange_order_response = self.ExchangeAPI.query_exchange_order_by_order_id(order_id)
        logger.debug("response body: %s", query_exchange_order_response.to_json())
        self.assertEqual('OK', query_exchange_order_response.code)



from dataclasses import dataclass
from enum import Enum
from typing import Optional

from dataclasses_json import dataclass_json


class TransferMode(Enum):
    BANK = "BANK"
    CRYPTO = "CRYPTO"
    INTERNAL = "INTERNAL"


@dataclass_json
@dataclass
class PaymentMethodDetail:
    asset_type: str


@dataclass_json
@dataclass
class PaymentMethod:
    title: str
    transfer_method: TransferMode
    detail: PaymentMethodDetail
    id: Optional[str] = None
    beneficiary_id: Optional[str] = None


@dataclass_json
@dataclass
class BankPaymentMethodDetail(PaymentMethodDetail):
    network: str
    account_number: str
    iban: Optional[str] = None
    bic: Optional[str] = None
    routing_number: Optional[str] = None
    bank_identifier: Optional[str] = None
    reference: Optional[str] = None
    bank_name: Optional[str] = None
    bank_address: Optional[str] = None
    bank_country: Optional[str] = None
    intermediary_bank_name: Optional[str] = None
    intermediary_account_number: Optional[str] = None
    intermediary_bic: Optional[str] = None


@dataclass_json
@dataclass
class CryptoPaymentMethodDetail(PaymentMethodDetail):
    network: str
    address: str


@dataclass_json
@dataclass
class InternalPaymentMethodDetail(PaymentMethodDetail):
    uid: str



from __future__ import absolute_import

import io
import json
import logging
import re
import ssl
from urllib.parse import urlencode

import certifi
import urllib3

from .api_exception import ApiException

logger = logging.getLogger(__name__)


class RESTResponse(io.IOBase):
    def __init__(self, resp):
        self.urllib3_response = resp
        self.status = resp.status
        self.reason = resp.reason
        self.data = resp.data

    def getheaders(self):
        """Returns a dictionary of the response headers."""
        return self.urllib3_response.headers

    def getheader(self, name, default=None):
        """Returns a given response header."""
        return self.urllib3_response.getheader(name, default)


class RESTClientObject:
    def __init__(self, configuration, pools_size=4, maxsize=None):
        self._setup_ssl_configuration(configuration)
        self._setup_pool_manager(configuration, pools_size, maxsize)

    def _setup_ssl_configuration(self, configuration):
        self.cert_reqs = ssl.CERT_REQUIRED if configuration.verify_ssl else ssl.CERT_NONE
        self.ca_certs = configuration.ssl_ca_cert or certifi.where()

    def _setup_pool_manager(self, configuration, pools_size, maxsize):
        additional_pool_args = {
            'assert_hostname': configuration.assert_hostname} if configuration.assert_hostname is not None else {}
        maxsize = maxsize or configuration.connection_pool_maxsize or 4

        if configuration.debug:
            logging.basicConfig(level=logging.DEBUG)

        self.pool_manager = self._create_pool_manager(configuration, pools_size, maxsize, additional_pool_args)

    def _create_pool_manager(self, configuration, pools_size, maxsize, additional_pool_args):
        if configuration.proxy:
            return urllib3.ProxyManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=self.cert_reqs,
                ca_certs=self.ca_certs,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                proxy_url=configuration.proxy,
                **additional_pool_args
            )
        else:
            return urllib3.PoolManager(
                num_pools=pools_size,
                maxsize=maxsize,
                cert_reqs=self.cert_reqs,
                ca_certs=self.ca_certs,
                cert_file=configuration.cert_file,
                key_file=configuration.key_file,
                **additional_pool_args
            )

    def request(self, method, url, query_params=None, headers=None, body=None, post_params=None, _preload_content=True,
                _request_timeout=None):
        method = method.upper()
        self._validate_request_method(method)

        if post_params and body:
            raise ValueError("body parameter cannot be used with post_params parameter.")

        headers = headers or {'Content-Type': 'application/json'}
        timeout = self._get_timeout(_request_timeout)

        try:
            return self._handle_request(method, url, query_params, headers, body, post_params, _preload_content,
                                        timeout)
        except urllib3.exceptions.SSLError as e:
            raise ApiException(status=0, reason=f"{type(e).__name__}\n{str(e)}")

    def _validate_request_method(self, method):
        valid_methods = ['GET', 'HEAD', 'DELETE', 'POST', 'PUT', 'PATCH', 'OPTIONS']
        if method not in valid_methods:
            raise ValueError(f"Invalid HTTP request method: {method}")

    def _get_timeout(self, _request_timeout):
        if not _request_timeout:
            return None
        if isinstance(_request_timeout, (int, float)):
            return urllib3.Timeout(total=_request_timeout)
        if isinstance(_request_timeout, tuple) and len(_request_timeout) == 2:
            return urllib3.Timeout(connect=_request_timeout[0], read=_request_timeout[1])
        raise ValueError("Invalid timeout format")

    def _handle_request(self, method, url, query_params, headers, body, post_params, _preload_content, timeout):
        if method in ['POST', 'PUT', 'PATCH', 'OPTIONS', 'DELETE']:
            return self._handle_body_methods(method, url, query_params, headers, body, post_params, _preload_content,
                                             timeout)
        else:
            return self._handle_query_methods(method, url, query_params, headers, _preload_content, timeout)

    def _handle_body_methods(self, method, url, query_params, headers, body, post_params, _preload_content, timeout):
        if query_params:
            url += '?' + urlencode(query_params)
        if re.search('json', headers['Content-Type'], re.IGNORECASE):
            request_body = json.dumps(body) if body is not None else '{}'
            return self.pool_manager.request(method, url, body=request_body, preload_content=_preload_content,
                                             timeout=timeout, headers=headers)
        if headers['Content-Type'] == 'application/x-www-form-urlencoded':
            return self.pool_manager.request(method, url, fields=post_params, encode_multipart=False,
                                             preload_content=_preload_content, timeout=timeout, headers=headers)
        if headers['Content-Type'] == 'multipart/form-data':
            del headers['Content-Type']
            return self.pool_manager.request(method, url, fields=post_params, encode_multipart=True,
                                             preload_content=_preload_content, timeout=timeout, headers=headers)
        if isinstance(body, str):
            return self.pool_manager.request(method, url, body=body, preload_content=_preload_content, timeout=timeout,
                                             headers=headers)
        raise ApiException(status=0,
                           reason="Cannot prepare a request message for provided arguments. Please check that your arguments match declared content type.")

    def _handle_query_methods(self, method, url, query_params, headers, _preload_content, timeout):
        response = self.pool_manager.request(method, url, fields=query_params, preload_content=_preload_content,
                                             timeout=timeout, headers=headers)
        if _preload_content:
            response = RESTResponse(response)
            logger.debug("response body: %s", response.data)
        if not 200 <= response.status <= 299:
            raise ApiException(http_resp=response)
        return response

    def GET(self, url, headers=None, query_params=None, _preload_content=True, _request_timeout=None):
        return self.request("GET", url, headers=headers, query_params=query_params, _preload_content=_preload_content,
                            _request_timeout=_request_timeout)

    def HEAD(self, url, headers=None, query_params=None, _preload_content=True, _request_timeout=None):
        return self.request("HEAD", url, headers=headers, query_params=query_params, _preload_content=_preload_content,
                            _request_timeout=_request_timeout)

    def OPTIONS(self, url, headers=None, query_params=None, post_params=None, body=None, _preload_content=True,
                _request_timeout=None):
        return self.request("OPTIONS", url, headers=headers, query_params=query_params, post_params=post_params,
                            _preload_content=_preload_content, _request_timeout=_request_timeout, body=body)

    def DELETE(self, url, headers=None, query_params=None, body=None, _preload_content=True, _request_timeout=None):
        return self.request("DELETE", url, headers=headers, query_params=query_params,
                            _preload_content=_preload_content, _request_timeout=_request_timeout, body=body)

    def POST(self, url, headers=None, query_params=None, post_params=None, body=None, _preload_content=True,
             _request_timeout=None):
        return self.request("POST", url, headers=headers, query_params=query_params, post_params=post_params,
                            _preload_content=_preload_content, _request_timeout=_request_timeout, body=body)

    def PUT(self, url, headers=None, query_params=None, post_params=None, body=None, _preload_content=True,
            _request_timeout=None):
        return self.request("PUT", url, headers=headers, query_params=query_params, post_params=post_params,
                            _preload_content=_preload_content, _request_timeout=_request_timeout, body=body)

    def PATCH(self, url, headers=None, query_params=None, post_params=None, body=None, _preload_content=True,
              _request_timeout=None):
        return self.request("PATCH", url, headers=headers, query_params=query_params, post_params=post_params,
                            _preload_content=_preload_content, _request_timeout=_request_timeout, body=body)



from __future__ import absolute_import

import logging
from typing import Optional

import pytest

from test.test_base_client import TestBaseClient
from unipayment.models import Beneficiary, BeneficiaryType, Relationship, PaymentMethod, TransferMode, \
    InternalPaymentMethodDetail, QueryBeneficiariesResponse, QueryPaymentMethodsResponse

logger = logging.getLogger(__name__)


def get_beneficiary_id(response: QueryBeneficiariesResponse, email) -> Optional[str]:
    for beneficiary in response.data.models:
        if beneficiary.get('email') == email:
            return beneficiary.get('id')
    return None


def get_payment_method_id(response: QueryPaymentMethodsResponse, transfer_method: TransferMode) -> Optional[str]:
    for payment_method in response.data:
        if transfer_method != payment_method.transfer_method:
            continue
        return payment_method.id
    return None


class TestBeneficiaryAPI(TestBaseClient):
    pass

    @pytest.mark.order(1)
    def test_create_beneficiary(self):
        """
            Test case create_beneficiary
        """

        beneficiary = Beneficiary(
            name="John Doe",
            email="john.doe@example.com",
            type=BeneficiaryType.INDIVIDUAL,
            relationship=Relationship.CUSTOMER,
            address="123 Main St",
            city="Springfield",
            state="IL",
            country="US",
            zip_code="62704"
        )
        create_beneficiary_response = self.BeneficiaryAPI.create_beneficiary(beneficiary)
        logger.debug("response body: %s", create_beneficiary_response.to_json())
        self.assertEqual('OK', create_beneficiary_response.code)
        self.assertIsNotNone(create_beneficiary_response.data.id)

    @pytest.mark.order(2)
    def test_query_beneficiaries(self):
        """
            Test case query_beneficiaries
        """
        query_beneficiaries_response = self.BeneficiaryAPI.query_beneficiaries()
        logger.debug("response body: %s", query_beneficiaries_response.to_json())
        self.assertEqual('OK', query_beneficiaries_response.code)
        self.assertIsNotNone(query_beneficiaries_response.data)

    @pytest.mark.order(3)
    def test_get_beneficiary_by_id(self):
        """
            Test case get_beneficiary_by_id
        """
        query_beneficiaries_response = self.BeneficiaryAPI.query_beneficiaries()
        beneficiary_id = get_beneficiary_id(query_beneficiaries_response, 'john.doe@example.com')

        beneficiary_response = self.BeneficiaryAPI.get_beneficiary_by_id(beneficiary_id)
        logger.debug("response body: %s", beneficiary_response.to_json())
        self.assertEqual('OK', beneficiary_response.code)
        self.assertIsNotNone(beneficiary_response.data.id)

    @pytest.mark.order(4)
    def test_create_payment_method(self):
        """
            Test case create_payment_method
        """
        query_beneficiaries_response = self.BeneficiaryAPI.query_beneficiaries()
        beneficiary_id = get_beneficiary_id(query_beneficiaries_response, 'john.doe@example.com')

        payment_method_detail = InternalPaymentMethodDetail(asset_type='USDT', uid='1000000')
        internal_payment_method = PaymentMethod(title='internal', transfer_method=TransferMode.INTERNAL,
                                                detail=payment_method_detail)
        payment_method_response = self.BeneficiaryAPI.create_payment_method(beneficiary_id,
                                                                            internal_payment_method)
        logger.debug("response body: %s", payment_method_response.to_json())
        self.assertEqual('OK', payment_method_response.code)

    @pytest.mark.order(5)
    def test_query_payment_methods(self):
        """
            Test case query_payment_methods
        """
        query_beneficiaries_response = self.BeneficiaryAPI.query_beneficiaries()
        beneficiary_id = get_beneficiary_id(query_beneficiaries_response, 'john.doe@example.com')

        query_payment_methods_response = self.BeneficiaryAPI.query_payment_methods(beneficiary_id)
        logger.debug("response body: %s", query_payment_methods_response.to_json())
        self.assertEqual('OK', query_payment_methods_response.code)

    @pytest.mark.order(6)
    def test_get_payment_method_by_id(self):
        """
            Test case query_payment_methods
        """
        query_beneficiaries_response = self.BeneficiaryAPI.query_beneficiaries()
        beneficiary_id = get_beneficiary_id(query_beneficiaries_response, 'john.doe@example.com')

        query_payment_methods_response = self.BeneficiaryAPI.query_payment_methods(beneficiary_id)
        payment_method_id = get_payment_method_id(query_payment_methods_response, TransferMode.INTERNAL)

        get_payment_method_response = self.BeneficiaryAPI.get_payment_method_by_id(beneficiary_id,
                                                                                   payment_method_id)
        logger.debug("response body: %s", get_payment_method_response.to_json())
        self.assertEqual('OK', get_payment_method_response.code)

    @pytest.mark.order(7)
    def delete_payment_method_by_id(self):
        """
            Test case query_payment_methods
        """
        query_beneficiaries_response = self.BeneficiaryAPI.query_beneficiaries()
        beneficiary_id = get_beneficiary_id(query_beneficiaries_response, 'john.doe@example.com')

        query_payment_methods_response = self.BeneficiaryAPI.query_payment_methods(beneficiary_id)
        payment_method_id = get_payment_method_id(query_payment_methods_response, TransferMode.INTERNAL)

        delete_payment_method_response = self.BeneficiaryAPI.delete_payment_method_by_id(
            beneficiary_id,
            payment_method_id)
        logger.debug("response body: %s", delete_payment_method_response.to_json())
        self.assertEqual('OK', delete_payment_method_response.code)

    @pytest.mark.order(8)
    def test_delete_beneficiary_by_id(self):
        """
            Test case delete_beneficiary_by_id
        """
        query_beneficiaries_response = self.BeneficiaryAPI.query_beneficiaries()
        beneficiary_id = get_beneficiary_id(query_beneficiaries_response, 'john.doe@example.com')
        beneficiary_response = self.BeneficiaryAPI.delete_beneficiary_by_id(beneficiary_id)
        logger.debug("response body: %s", beneficiary_response.to_json())
        self.assertEqual('OK', beneficiary_response.code)
        self.assertIsNone(beneficiary_response.data)



class QueryExchangeOrdersRequest(object):
    def __init__(self, page_no=1, page_size=10, is_asc=True, from_currency=None, to_currency=None):
        self.page_no = page_no
        self.page_size = page_size
        self.is_asc = is_asc
        self.from_currency = from_currency
        self.to_currency = to_currency

    def to_str(self):
        query_params = {
            "page_no": self.page_no,
            "page_size": self.page_size,
            "is_asc": str(self.is_asc).lower()
        }
        if self.from_currency is not None:
            query_params["from_currency"] = self.from_currency
        if self.to_currency is not None:
            query_params["to_currency"] = self.to_currency
        return query_params



from __future__ import absolute_import

import logging
from typing import Optional

from test.test_base_client import TestBaseClient
from unipayment.models import GetWalletAccountsResponse

logger = logging.getLogger(__name__)


def get_account_id(response: GetWalletAccountsResponse, asset_type) -> Optional[str]:
    for balance in response.data:
        if balance.asset_type == asset_type:
            return balance.id
    return None


class TestWalletAPI(TestBaseClient):

    def test_get_balances(self):
        """
            Test case get_balances
        """
        get_wallet_balances_response = self.WalletAPI.get_balances()
        logger.debug("response body: %s", get_wallet_balances_response.to_json())
        self.assertEqual('OK', get_wallet_balances_response.code)

    def test_get_accounts(self):
        """
            Test case get_balances
        """
        get_wallet_accounts_response = self.WalletAPI.get_accounts()
        logger.debug("response body: %s", get_wallet_accounts_response.to_json())
        self.assertEqual('OK', get_wallet_accounts_response.code)

    def test_get_deposit_bank_account(self):
        """
            Test case get_deposit_bank_account
        """
        get_wallet_accounts_response = self.WalletAPI.get_accounts()
        fiat_account_id = get_account_id(get_wallet_accounts_response, 'USD')
        get_deposit_bank_account_response = self.WalletAPI.get_deposit_bank_account(fiat_account_id)
        logger.debug("response body: %s", get_deposit_bank_account_response.to_json())
        self.assertEqual('OK', get_deposit_bank_account_response.code)
        self.assertIsNotNone(get_deposit_bank_account_response.data)

    def test_get_deposit_address(self):
        """
            Test case get_deposit_address
        """
        get_wallet_accounts_response = self.WalletAPI.get_accounts()
        crypto_account_id = get_account_id(get_wallet_accounts_response, 'BNB')
        get_deposit_address_response = self.WalletAPI.get_deposit_address(crypto_account_id)
        logger.debug("response body: %s", get_deposit_address_response.to_json())
        self.assertEqual('OK', get_deposit_address_response.code)
        self.assertIsNotNone(get_deposit_address_response.data)

    def test_query_transactions(self):
        """
            Test case query_transactions
        """
        get_wallet_accounts_response = self.WalletAPI.get_accounts()
        fiat_account_id = get_account_id(get_wallet_accounts_response, 'USD')
        query_wallet_account_transactions_response = self.WalletAPI.query_transactions(fiat_account_id)
        logger.debug("response body: %s", query_wallet_account_transactions_response.to_json())
        self.assertEqual('OK', query_wallet_account_transactions_response.code)



from unipayment.models import PaymentStatus
from unipayment.models import TransferMode


class QueryPaymentsRequest(object):
    def __init__(self, page_no=1,
                 page_size=10,
                 is_asc=True,
                 from_account_id=None,
                 to_account_id=None,
                 transfer_method: TransferMode = None,
                 payment_method_id=None,
                 crypto_address=None,
                 bank_account=None,
                 asset_type=None,
                 status: PaymentStatus = None):
        self.page_no = page_no
        self.page_size = page_size
        self.is_asc = is_asc
        self.from_account_id = from_account_id
        self.to_account_id = to_account_id
        self.transfer_method = transfer_method
        self.payment_method_id = payment_method_id
        self.crypto_address = crypto_address
        self.bank_account = bank_account
        self.asset_type = asset_type
        self.status = status

    def to_str(self):
        query_params = {
            "page_no": self.page_no,
            "page_size": self.page_size,
            "is_asc": str(self.is_asc).lower()
        }

        if self.from_account_id is not None:
            query_params['from_account_id'] = self.from_account_id
        if self.to_account_id is not None:
            query_params['to_account_id'] = self.to_account_id
        if self.transfer_method is not None:
            query_params['transfer_method'] = self.transfer_method
        if self.payment_method_id is not None:
            query_params['payment_method_id'] = self.payment_method_id
        if self.crypto_address is not None:
            query_params['crypto_address'] = self.crypto_address
        if self.from_account_id is not None:
            query_params['bank_account'] = self.bank_account
        if self.asset_type is not None:
            query_params['asset_type'] = self.asset_type
        if self.status is not None:
            query_params['status'] = self.status

        return query_params



class ApiException(Exception):
    def __init__(self, status=None, reason=None, http_resp=None):
        self.status = http_resp.status if http_resp else status
        self.reason = http_resp.reason if http_resp else reason
        self.body = http_resp.data if http_resp else None
        self.headers = http_resp.headers if http_resp else None

    def __str__(self):
        error_message = f"({self.status})\nReason: {self.reason}\n"
        if self.headers:
            error_message += f"HTTP response headers: {self.headers}\n"
        if self.body:
            error_message += f"HTTP response body: {self.body}\n"
        return error_message



from dataclasses import dataclass
from enum import Enum

from dataclasses_json import dataclass_json


class TransactionType(Enum):
    DEPOSIT = "DEPOSIT"
    DEPOSITPAYMENT = "DEPOSITPAYMENT"
    PAYMENTFEE = "PAYMENTFEE"
    FEEADJUSTMENT = "FEEADJUSTMENT"
    ADJUSTMENTSWAP = "ADJUSTMENTSWAP"
    SWAPINVOICE = "SWAPINVOICE"
    INVOICEREFUND = "INVOICEREFUND"
    REFUND = "REFUND"


@dataclass_json
@dataclass
class QueryWalletAccountTransactionsRequest:
    def __init__(self, page_no=1, page_size=20, is_asc=True, txn_type: TransactionType = None):
        self.page_no = page_no
        self.page_size = page_size
        self.is_asc = is_asc
        self.txn_type = txn_type

    def to_str(self):
        query_params = {
            "page_no": self.page_no,
            "page_size": self.page_size,
            "is_asc": str(self.is_asc).lower()
        }
        if self.txn_type:
            query_params["txn_type"] = self.txn_type.value
        return query_params



from dataclasses import dataclass
from enum import Enum
from typing import Optional

from dataclasses_json import dataclass_json


class BeneficiaryType(Enum):
    INDIVIDUAL = "INDIVIDUAL"
    COMPANY = "COMPANY"
    CORPORATE = "CORPORATE"
    UNKNOWN = "UNKNOWN"


class Relationship(Enum):
    BRANCHOFFICE = "BRANCHOFFICE"
    SPOUSE = "SPOUSE"
    BUSINESSPARTNER = "BUSINESSPARTNER"
    PARENT = "PARENT"
    FRIEND = "FRIEND"
    HOLDINGCOMPANY = "HOLDINGCOMPANY"
    SUPPLIER = "SUPPLIER"
    EXSPOUSE = "EXSPOUSE"
    CHILDREN = "CHILDREN"
    FRANCHISEE = "FRANCHISEE"
    NOTRELATED = "NOTRELATED"
    SELF = "SELF"
    CUSTOMER = "CUSTOMER"
    EMPLOYEE = "EMPLOYEE"
    SUBSIDIARYCOMPANY = "SUBSIDIARYCOMPANY"
    CREDITOR = "CREDITOR"
    DEBTOR = "DEBTOR"
    SIBLING = "SIBLING"
    RELATIVE = "RELATIVE"


@dataclass_json
@dataclass
class Beneficiary:
    name: str
    address: Optional[str] = None
    email: Optional[str] = None
    type: Optional[BeneficiaryType] = None
    relationship: Optional[Relationship] = None
    id: Optional[str] = None
    city: Optional[str] = None
    state: Optional[str] = None
    country: Optional[str] = None
    zip_code: Optional[str] = None



import unittest

from unipayment import WebhookSignatureUtil


class TestWebhookSignatureUtil(unittest.TestCase):

    def test_is_valid(self):
        """
            Test case WebhookSignatureUtil
            Use raw json payload (no formatting or pretty print)
        """
        payload = ('{"ipn_type":"invoice","event":"invoice_created","app_id":"2a9bd90b-fe95-4659-83cb-04de662fbbac",'
                   '"payment_method_type":"Unknown","invoice_id":"Gy3TVe9Fbe5idGbez19pMr",'
                   '"order_id":"b49f70e5-a0da-4450-ae86-69b0a81708cc","price_amount":1.0,"price_currency":"USD",'
                   '"network":null,"address":null,"pay_currency":null,"pay_amount":0.0,"exchange_rate":0.0,'
                   '"paid_amount":0.0,"confirmed_amount":0.0,"refunded_price_amount":0.0,'
                   '"create_time":"2024-08-16T17:35:51.4194631Z","expiration_time":"2024-08-16T17:55:51.4195145Z",'
                   '"status":"New","error_status":"None","ext_args":"Merchant Pass Through Data","channel":null,'
                   '"notify_id":"b2fd88c1-3d34-4bd2-9f55-c3e5b3097173","notify_time":"2024-08-16T17:35:51.4443531Z"}')
        secret_key = 's3cretKey@2024%'
        signature = 'cWcyVMKMQS8M+zwlUxPuROjF3hLWzAL6TSvWKC8K3FE='
        valid = WebhookSignatureUtil.is_valid(payload, secret_key, signature)
        self.assertTrue(valid)



from __future__ import absolute_import

import platform
from multiprocessing.pool import ThreadPool

from .configuration import Configuration
from .rest import RESTClientObject

SDK_VERSION = '2.0.2'


class ApiClient(object):

    def __init__(self, configuration: Configuration):
        self.pool = ThreadPool()
        self.rest_client = RESTClientObject(configuration)
        self.default_headers = {}
        self.user_agent = ('unipayment_sdk_python/' + SDK_VERSION + ' (' + platform.system() + ' ' + platform.release()
                           + ')')

    def __del__(self):
        self.pool.close()
        self.pool.join()

    @property
    def user_agent(self):
        return self.default_headers['User-Agent']

    @user_agent.setter
    def user_agent(self, value):
        self.default_headers['User-Agent'] = value

    def set_default_header(self, header_name, header_value):
        self.default_headers[header_name] = header_value

    def request(self, url, method, query_params=None, headers=None, post_params=None, body=None, _preload_content=False,
                _request_timeout=None):
        if headers is None:
            headers = self.default_headers

        if method == "GET":
            return self.rest_client.GET(url, query_params=query_params, headers=headers,
                                        _preload_content=_preload_content, _request_timeout=_request_timeout)
        elif method == "HEAD":
            return self.rest_client.HEAD(url, query_params=query_params, headers=headers,
                                         _preload_content=_preload_content, _request_timeout=_request_timeout)
        elif method == "OPTIONS":
            return self.rest_client.OPTIONS(url, query_params=query_params, headers=headers,
                                            _preload_content=_preload_content, _request_timeout=_request_timeout)
        elif method == "POST":
            return self.rest_client.POST(url, headers=headers, post_params=post_params,
                                         _preload_content=_preload_content, _request_timeout=_request_timeout,
                                         body=body)
        elif method == "PUT":
            return self.rest_client.PUT(url, headers=headers, post_params=post_params,
                                        _preload_content=_preload_content, _request_timeout=_request_timeout, body=body)
        elif method == "PATCH":
            return self.rest_client.PATCH(url, headers=headers, post_params=post_params,
                                          _preload_content=_preload_content, _request_timeout=_request_timeout,
                                          body=body)
        elif method == "DELETE":
            return self.rest_client.DELETE(url, query_params=query_params, headers=headers,
                                           _preload_content=_preload_content, _request_timeout=_request_timeout,
                                           body=body)
        else:
            raise ValueError("HTTP method must be `GET`, `HEAD`, `OPTIONS`, `POST`, `PATCH`, `PUT` or `DELETE`.")



from .base_client import BaseClient
from .models.accept_quote_response import AcceptQuoteResponse
from .models.query_exchange_order_response import QueryExchangeOrderResponse
from .models.query_exchange_orders_request import QueryExchangeOrdersRequest
from .models.query_exchange_orders_response import QueryExchangeOrdersResponse
from .models.quote_request import QuoteRequest
from .models.quote_response import QuoteResponse


class ExchangeAPI(BaseClient):
    pass

    def get_quote(self, quote_request: QuoteRequest) -> QuoteResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/exchange/quote'
        response_text = self.call_api(url, 'GET', query_params=quote_request.to_str())
        return QuoteResponse.from_json(response_text)

    def accept_quote(self, quote_id) -> AcceptQuoteResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/exchange/quote/{quote_id}'
        response_text = self.call_api(url, 'PUT')
        return AcceptQuoteResponse.from_json(response_text)

    def query_exchange_orders(self,
                              query_exchange_orders_request: QueryExchangeOrdersRequest = None) -> QueryExchangeOrdersResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/exchange/orders'

        if query_exchange_orders_request is None:
            query_exchange_orders_request = QueryExchangeOrdersRequest()

        response_text = self.call_api(url, 'GET', query_params=query_exchange_orders_request.to_str())
        return QueryExchangeOrdersResponse.from_json(response_text)

    def query_exchange_order_by_order_id(self, exchange_order_id) -> QueryExchangeOrderResponse:
        url = f'{self.configuration.host}/v{self.configuration.api_version}/exchange/orders/{exchange_order_id}'

        response_text = self.call_api(url, 'GET')
        return QueryExchangeOrderResponse.from_json(response_text)



from setuptools import setup, find_packages

NAME = "unipayment"
VERSION = "2.0.2"
# To install the library, run the following
#
# python setup.py install
#
# prerequisite: setuptools
# http://pypi.python.org/pypi/setuptools


REQUIRES = ["urllib3 >= 2.2.2", "six >= 1.16.0", "certifi >= 2024.7.4", "python-dateutil >= 2.9.0",
            "setuptools >= 71.1.0", "dataclasses-json ~= 0.6.7"]

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setup(
    name=NAME,
    version=VERSION,
    description="UniPayment Python SDK",
    auther="UniPayment.io",
    auther_email="develop@unipayment.io",
    install_requires=REQUIRES,
    packages=find_packages(exclude=("test",)),
    include_package_data=True,
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/UniCryptoLab/UniPaymentClient.Python"
)



from dataclasses import dataclass
from typing import List, TypeVar, Generic

from dataclasses_json import dataclass_json

T = TypeVar('T')


@dataclass_json
@dataclass
class QueryResult(Generic[T]):
    models: List[T]
    page_no: int
    total: int
    page_count: int
    page_size: int



from __future__ import absolute_import

from . import models as models
from .rest import RESTResponse
from .api_client import ApiClient
from .api_exception import ApiException
from .unipayment_sdk_exception import UnipaymentSdkException
from .base_client import BaseClient
from .configuration import Configuration
from .beneficiary_api import BeneficiaryAPI
from .billing_api import BillingAPI
from .common_api import CommonAPI
from .exchange_api import ExchangeAPI
from .payment_api import PaymentAPI
from .wallet_api import WalletAPI
from .webhook_api import WebhookAPI
from .webhook_signature_util import WebhookSignatureUtil

__all__ = ['ApiClient', 'ApiException', 'BaseClient', 'Configuration', 'RESTResponse', 'UnipaymentSdkException',
           'BeneficiaryAPI', 'BillingAPI', 'CommonAPI', 'ExchangeAPI', 'PaymentAPI', 'WalletAPI', 'WebhookAPI',
           'WebhookSignatureUtil', 'models']



from dataclasses import dataclass
from typing import Optional

from dataclasses_json import dataclass_json

from unipayment.models import BankInfo
from unipayment.models import Beneficiary


@dataclass_json
@dataclass
class DepositBankAccount:
    network: str
    account_number: str
    iban: Optional[str] = None
    bic: Optional[str] = None
    routing_number: Optional[str] = None
    beneficiary: Optional[Beneficiary] = None
    bank_info: Optional[BankInfo] = None
    reference: Optional[str] = None



from dataclasses import dataclass
from typing import List

from dataclasses_json import dataclass_json

from unipayment.models import WalletBalance


@dataclass_json
@dataclass
class GetWalletAccountsResponse:
    code: str
    msg: str
    data: List[WalletBalance]



from dataclasses import dataclass

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class PaymentFee:
    asset_type: str
    network: str
    fee_type: str
    fee_rate: float
    flat_rate: float
    min_txn_fee: float
    max_txn_fee: float



from __future__ import absolute_import

import logging

from test.test_base_client import TestBaseClient
from unipayment.models import UpdateNotifyURLRequest, UpdateSecretKeyRequest

logger = logging.getLogger(__name__)


class TestWebhookAPI(TestBaseClient):
    pass

    def test_update_notify_url(self):
        """
            Test case update_notify_url
        """

        update_notify_url_request = UpdateNotifyURLRequest(notify_url='https://en7exsmaa68jo.x.pipedream.net')
        update_notify_url_response = self.WebhookAPI.update_notify_url(update_notify_url_request)
        logger.debug("response body: %s", update_notify_url_response.to_json())
        self.assertEqual('OK', update_notify_url_response.code)

    def test_update_secret_key(self):
        """
            Test case update_secret_key
        """

        update_secret_key_request = UpdateSecretKeyRequest(secret_key='s3cretKey@2024%')
        update_secret_key_response = self.WebhookAPI.update_secret_key(update_secret_key_request)
        logger.debug("response body: %s", update_secret_key_response.to_json())
        self.assertEqual('OK', update_secret_key_response.code)



from dataclasses import dataclass

from dataclasses_json import dataclass_json


@dataclass_json
@dataclass
class ExchangeRate:
    to_currency: str
    from_currency: str
    rate: float
    ask: float
    bid: float



from __future__ import absolute_import

import logging
import uuid

import pytest

from test.test_base_client import TestBaseClient
from unipayment.models import CreateInvoiceRequest, QueryInvoicesRequest, BuyerInfo

logger = logging.getLogger(__name__)


class TestBillingAPI(TestBaseClient):
    pass

    @pytest.mark.order(1)
    def test_create_invoice(self):
        """
            Test case create_invoice
        """
        order_id = uuid.uuid4()
        create_invoice_request = CreateInvoiceRequest(app_id=self.configuration.app_id, price_amount=20.0,
                                                      price_currency='USD', order_id=order_id, lang='en',
                                                      ext_args='"Merchant Pass Through Data')
        create_invoice_response = self.BillingAPI.create_invoice(create_invoice_request)
        logger.debug("response body: %s", create_invoice_response)
        self.assertEqual('OK', create_invoice_response.code)

    @pytest.mark.order(2)
    def test_query_invoices(self):
        """
            Test case query_invoices
        """

        query_invoices_request = QueryInvoicesRequest(app_id=self.configuration.app_id)
        query_invoices_response = self.BillingAPI.query_invoices(query_invoices_request)
        logger.debug("response body: %s", query_invoices_response)
        self.assertEqual('OK', query_invoices_response.code)

    @pytest.mark.order(3)
    def test_get_invoice_by_id(self):
        """
            Test case get_invoice_by_id
        """

        query_invoices_request = QueryInvoicesRequest(app_id=self.configuration.app_id)
        query_invoices_response = self.BillingAPI.query_invoices(query_invoices_request)
        invoice_id = query_invoices_response.data.models[0].get("invoice_id")

        get_invoice_by_id_response = self.BillingAPI.get_invoice_by_id(invoice_id)
        logger.debug("response body: %s", get_invoice_by_id_response)
        self.assertEqual('OK', get_invoice_by_id_response.code)

    @pytest.mark.order(4)
    def test_create_invoice_host_to_host(self):
        """
            Test case create_invoice
        """
        order_id = uuid.uuid4()
        create_invoice_request = CreateInvoiceRequest(app_id=self.configuration.app_id, price_amount=20.0,
                                                      price_currency='USD', order_id=order_id, lang='en',
                                                      ext_args='"Merchant Pass Through Data', host_to_host_mode=True,
                                                      payment_method_type='CRYPTO', pay_currency='BNB',
                                                      network='NETWORK_BSC')
        create_invoice_response = self.BillingAPI.create_invoice(create_invoice_request)
        logger.debug("response body: %s", create_invoice_response)
        self.assertEqual('OK', create_invoice_response.code)

    @pytest.mark.order(4)
    def test_create_invoice_buyer_info(self):
        """
            Test case create_invoice
        """
        order_id = uuid.uuid4()
        buyer_info = BuyerInfo(name='John Doe', email='john@doe.com', phone='555-555-5555', address1='123 Main Street',
                               address2="", state='NYC', country='US', zip_code='12345', city='San Francisco', )
        create_invoice_request = CreateInvoiceRequest(app_id=self.configuration.app_id, price_amount=20.0,
                                                      price_currency='USD', order_id=order_id, lang='en',
                                                      ext_args='"Merchant Pass Through Data', buyer_info=buyer_info)
        create_invoice_response = self.BillingAPI.create_invoice(create_invoice_request)
        logger.debug("response body: %s", create_invoice_response)
        self.assertEqual('OK', create_invoice_response.code)



from dataclasses import dataclass

from dataclasses_json import dataclass_json

from unipayment.models import ExchangeOrder
from unipayment.models import QueryResult


@dataclass_json
@dataclass
class QueryExchangeOrdersResponse:
    code: str
    msg: str
    data: QueryResult[ExchangeOrder]
